@@ -2,6 +2,7 @@
     Copyright 2002-2005 Roberto Raggi <roberto@kdevelop.org>
     Copyright 2006 Adam Treat <treat@kde.org>
     Copyright 2006-2008 Hamish Rodda <rodda@kde.org>
+    Copyright 2007-2008 David Nolden <david.nolden.kdevelop@art-master.de>
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
@@ -28,7 +29,7 @@ namespace KDevelop {
 REGISTER_TYPE(ConstantIntegralType);
 
 ConstantIntegralType::ConstantIntegralType(const ConstantIntegralType& rhs)
-  : IntegralType(copyData<ConstantIntegralTypeData>(*rhs.d_func()))
+  : IntegralType(copyData<ConstantIntegralType>(*rhs.d_func()))
 {
 }
 
@@ -38,9 +39,8 @@ ConstantIntegralType::ConstantIntegralType(ConstantIntegralTypeData& data)
 }
 
 ConstantIntegralType::ConstantIntegralType(uint type)
-  : IntegralType(createData<ConstantIntegralTypeData>())
+  : IntegralType(createData<ConstantIntegralType>())
 {
-  d_func_dynamic()->setTypeClassId<ConstantIntegralType>();
   setDataType(type);
   setModifiers(ConstModifier);
 }
@@ -117,32 +117,36 @@ uint ConstantIntegralType::hash() const
 template<>
 KDEVPLATFORMLANGUAGE_EXPORT
 void ConstantIntegralType::setValueInternal<qint64>(qint64 value) {
-  if((modifiers() & UnsignedModifier))
+  if((modifiers() & UnsignedModifier)) {
     kDebug() << "setValue(signed) called on unsigned type";
+  }
   d_func_dynamic()->m_value = value;
 }
 
 template<>
 KDEVPLATFORMLANGUAGE_EXPORT
 void ConstantIntegralType::setValueInternal<quint64>(quint64 value) {
-  if(!(modifiers() & UnsignedModifier))
+  if(!(modifiers() & UnsignedModifier)) {
     kDebug() << "setValue(unsigned) called on not unsigned type";
+  }
   d_func_dynamic()->m_value = (qint64)value;
 }
 
 template<>
 KDEVPLATFORMLANGUAGE_EXPORT
 void ConstantIntegralType::setValueInternal<float>(float value) {
-  if(dataType() != TypeFloat)
+  if(dataType() != TypeFloat) {
     kDebug() << "setValue(float) called on non-float type";
+  }
   memcpy(&d_func_dynamic()->m_value, &value, sizeof(float));
 }
 
 template<>
 KDEVPLATFORMLANGUAGE_EXPORT
 void ConstantIntegralType::setValueInternal<double>(double value) {
-  if(dataType() != TypeDouble)
+  if(dataType() != TypeDouble) {
     kDebug() << "setValue(double) called on non-double type";
+  }
   memcpy(&d_func_dynamic()->m_value, &value, sizeof(double));
 }
 
