@@ -40,12 +40,15 @@
 #include <interfaces/ilanguage.h>
 #include <interfaces/iuicontroller.h>
 #include <interfaces/ilanguagecontroller.h>
+#include <interfaces/contextmenuextension.h>
+#include <language/interfaces/codecontext.h>
 #include <language/duchain/duchainlock.h>
 #include <language/duchain/duchain.h>
 #include <language/duchain/ducontext.h>
 #include <language/duchain/declaration.h>
 #include <language/duchain/use.h>
 #include <language/duchain/duchainutils.h>
+#include <language/duchain/dumpchain.h>
 #include <language/duchain/functiondefinition.h>
 #include <language/duchain/parsingenvironment.h>
 #include <language/interfaces/ilanguagesupport.h>
@@ -56,6 +59,9 @@
 #include <language/duchain/specializationstore.h>
 #include <language/util/navigationtooltip.h>
 #include <language/duchain/navigation/abstractnavigationwidget.h>
+#include <language/interfaces/iquickopen.h>
+#include <interfaces/iplugincontroller.h>
+#include <sublime/mainwindow.h>
 
 const unsigned int highlightingTimeout = 150;
 
@@ -66,7 +72,7 @@ using KTextEditor::View;
 
 bool toolTipEnabled = true;
 
-static QWidget* masterWidget(QWidget* w) {
+QWidget* masterWidget(QWidget* w) {
   while(w && w->parent() && qobject_cast<QWidget*>(w->parent()))
     w = qobject_cast<QWidget*>(w->parent());
   return w;
@@ -77,9 +83,9 @@ class ContextBrowserViewFactory: public KDevelop::IToolViewFactory
 public:
     ContextBrowserViewFactory(ContextBrowserPlugin *plugin): m_plugin(plugin) {}
 
-    virtual QWidget* create(QWidget */*parent*/ = 0)
+    virtual QWidget* create(QWidget *parent = 0)
     {
-        ContextBrowserView* ret = new ContextBrowserView(m_plugin);
+        ContextBrowserView* ret = new ContextBrowserView(m_plugin, parent);
         QObject::connect(ret, SIGNAL(startDelayedBrowsing(KTextEditor::View*)), m_plugin, SLOT(startDelayedBrowsing(KTextEditor::View*)));
         QObject::connect(ret, SIGNAL(stopDelayedBrowsing()), m_plugin, SLOT(stopDelayedBrowsing()));
         return ret;
@@ -99,17 +105,72 @@ private:
     ContextBrowserPlugin *m_plugin;
 };
 
+void ContextBrowserPlugin::createActionsForMainWindow(Sublime::MainWindow* window, QString& xmlFile, KActionCollection& actions) {
+    xmlFile = "kdevcontextbrowser.rc" ;
+
+    KAction* previousContext = actions.addAction("previous_context");
+    previousContext->setText( i18n("&Previous Visited Context") );
+    previousContext->setIcon( KIcon("go-previous-context" ) );
+    previousContext->setShortcut( Qt::META | Qt::Key_Left );
+    QObject::connect(previousContext, SIGNAL(triggered(bool)), this, SLOT(previousContextShortcut()));
+
+    KAction* nextContext = actions.addAction("next_context");
+    nextContext->setText( i18n("&Next Visited Context") );
+    nextContext->setIcon( KIcon("go-next-context" ) );
+    nextContext->setShortcut( Qt::META | Qt::Key_Right );
+    QObject::connect(nextContext, SIGNAL(triggered(bool)), this, SLOT(nextContextShortcut()));
+
+    KAction* previousUse = actions.addAction("previous_use");
+    previousUse->setText( i18n("&Previous Use") );
+    previousUse->setIcon( KIcon("go-previous-use") );
+    previousUse->setShortcut( Qt::META | Qt::SHIFT |  Qt::Key_Left );
+    QObject::connect(previousUse, SIGNAL(triggered(bool)), this, SLOT(previousUseShortcut()));
+
+    KAction* nextUse = actions.addAction("next_use");
+    nextUse->setText( i18n("&Next Use") );
+    nextUse->setIcon( KIcon("go-next-use") );
+    nextUse->setShortcut( Qt::META | Qt::SHIFT | Qt::Key_Right );
+    QObject::connect(nextUse, SIGNAL(triggered(bool)), this, SLOT(nextUseShortcut()));
+
+    KAction* outline = actions.addAction("outline_line");
+    outline->setText(i18n("Context Browser"));
+    QWidget* w = toolbarWidgetForMainWindow(window);
+    w->setHidden(false);
+    outline->setDefaultWidget(w);
+}
+
+
+void ContextBrowserPlugin::nextContextShortcut()
+{
+  foreach(ContextBrowserView* view, m_views)
+  {
+    kDebug() << masterWidget(ICore::self()->uiController()->activeMainWindow()) << masterWidget(view);
+    if(masterWidget(ICore::self()->uiController()->activeMainWindow()) == masterWidget(view)) {
+      view->historyNext();
+      return;
+    }
+  }
+}
+
+void ContextBrowserPlugin::previousContextShortcut()
+{
+  foreach(ContextBrowserView* view, m_views)
+  {
+    if(masterWidget(ICore::self()->uiController()->activeMainWindow()) == masterWidget(view)) {
+      view->historyPrevious();
+      return;
+    }
+  }
+}
+
 K_PLUGIN_FACTORY(ContextBrowserFactory, registerPlugin<ContextBrowserPlugin>(); )
 K_EXPORT_PLUGIN(ContextBrowserFactory(KAboutData("kdevcontextbrowser","kdevcontextbrowser",ki18n("Context Browser"), "0.1", ki18n("Shows information for the current context"), KAboutData::License_GPL)))
 
 ContextBrowserPlugin::ContextBrowserPlugin(QObject *parent, const QVariantList&)
     : KDevelop::IPlugin(ContextBrowserFactory::componentData(), parent)
-    , m_viewFactory(new ContextBrowserViewFactory(this))
+    , m_backupsMutex(QMutex::Recursive)
+    , m_viewFactory(new ContextBrowserViewFactory(this)), m_lastInsertionDocument(0)
 {
-  setXMLFile( "kdevcontextbrowser.rc" );
-
-  KActionCollection* actions = actionCollection();
-
   core()->uiController()->addToolView(i18n("Code Browser"), m_viewFactory);
 
   connect( core()->documentController(), SIGNAL( textDocumentCreated( KDevelop::IDocument* ) ), this, SLOT( textDocumentCreated( KDevelop::IDocument* ) ) );
@@ -122,26 +183,10 @@ ContextBrowserPlugin::ContextBrowserPlugin(QObject *parent, const QVariantList&)
   m_updateTimer = new QTimer(this);
   m_updateTimer->setSingleShot(true);
   connect( m_updateTimer, SIGNAL( timeout() ), this, SLOT( updateViews() ) );
-
-  KAction* previousContext = actions->addAction("previous_context");
-  previousContext->setText( i18n("&Previous Visited Context") );
-  previousContext->setShortcut( Qt::META | Qt::Key_Left );
-  connect(previousContext, SIGNAL(triggered(bool)), this, SIGNAL(previousContextShortcut()));
-
-  KAction* nextContext = actions->addAction("next_context");
-  nextContext->setText( i18n("&Next Visited Context") );
-  nextContext->setShortcut( Qt::META | Qt::Key_Right );
-  connect(nextContext, SIGNAL(triggered(bool)), this, SIGNAL(nextContextShortcut()));
-
-  KAction* previousUse = actions->addAction("previous_use");
-  previousUse->setText( i18n("&Previous Use") );
-  previousUse->setShortcut( Qt::META | Qt::SHIFT |  Qt::Key_Left );
-  connect(previousUse, SIGNAL(triggered(bool)), this, SLOT(previousUseShortcut()));
-
-  KAction* nextUse = actions->addAction("next_use");
-  nextUse->setText( i18n("&Next Use") );
-  nextUse->setShortcut( Qt::META | Qt::SHIFT | Qt::Key_Right );
-  connect(nextUse, SIGNAL(triggered(bool)), this, SLOT(nextUseShortcut()));
+  
+  //Needed global action for the context-menu extensions
+  m_findUses = new QAction(i18n("Find Uses"), this);
+  connect(m_findUses, SIGNAL(triggered(bool)), this, SLOT(findUses()));
 }
 
 ContextBrowserPlugin::~ContextBrowserPlugin()
@@ -152,6 +197,10 @@ ContextBrowserPlugin::~ContextBrowserPlugin()
 
 void ContextBrowserPlugin::watchRange(KTextEditor::SmartRange* range)
 {
+  KTextEditor::SmartInterface* smart = dynamic_cast<KTextEditor::SmartInterface*>(range->document());
+  Q_ASSERT(smart);
+  QMutexLocker lock(smart->smartMutex());
+
   if (range->watchers().contains( this ))
     return;
 
@@ -174,6 +223,7 @@ void ContextBrowserPlugin::unload()
 }
 
 void ContextBrowserPlugin::rangeDeleted( KTextEditor::SmartRange *range ) {
+  QMutexLocker lock(&m_backupsMutex);
   m_backups.remove( range );
   m_watchedRanges.remove(range);
 
@@ -198,6 +248,49 @@ void ContextBrowserPlugin::mouseExitedRange( KTextEditor::SmartRange* /*range*/,
   m_updateTimer->start(1); // triggers updateViews()
 }
 
+KDevelop::ContextMenuExtension ContextBrowserPlugin::contextMenuExtension(KDevelop::Context* context)
+{
+  KDevelop::ContextMenuExtension menuExt = KDevelop::IPlugin::contextMenuExtension( context );
+
+  KDevelop::DeclarationContext *codeContext = dynamic_cast<KDevelop::DeclarationContext*>(context);
+
+  if (!codeContext)
+      return menuExt;
+
+  DUChainReadLocker lock(DUChain::lock());
+  
+  if(!codeContext->declaration().data())
+    return menuExt;
+  
+  qRegisterMetaType<KDevelop::IndexedDeclaration>("KDevelop::IndexedDeclaration");
+  
+  m_findUses->setData(QVariant::fromValue(codeContext->declaration()));
+  menuExt.addAction(KDevelop::ContextMenuExtension::ExtensionGroup, m_findUses);
+
+  return menuExt;
+}
+
+void ContextBrowserPlugin::findUses()
+{
+  QAction* action = qobject_cast<QAction*>(sender());
+  Q_ASSERT(action);
+  DUChainReadLocker lock(DUChain::lock());
+  
+  KDevelop::IndexedDeclaration decl = action->data().value<KDevelop::IndexedDeclaration>();
+  if(!decl.data())
+    return;
+  QWidget* widget = ICore::self()->uiController()->findToolView(i18n("Code Browser"), m_viewFactory, KDevelop::IUiController::CreateAndRaise);
+  if(!widget)
+    return;
+  ContextBrowserView* view = dynamic_cast<ContextBrowserView*>(widget);
+  Q_ASSERT(view);
+  view->allowLockedUpdate();
+  view->setDeclaration(decl.data(), decl.data()->topContext(), true);
+  KDevelop::AbstractNavigationWidget* navigationWidget = dynamic_cast<KDevelop::AbstractNavigationWidget*>(view->navigationWidget());
+  if(navigationWidget)
+    navigationWidget->executeContextAction("show_uses");
+}
+
 void ContextBrowserPlugin::textHintRequested(const KTextEditor::Cursor& cursor, QString&) {
   m_mouseHoverCursor = SimpleCursor(cursor);
   View* view = dynamic_cast<View*>(sender());
@@ -214,8 +307,10 @@ void ContextBrowserPlugin::textHintRequested(const KTextEditor::Cursor& cursor,
 }
 
 void ContextBrowserPlugin::stopDelayedBrowsing() {
-  if(m_currentToolTip)
-    delete m_currentToolTip;
+  if(m_currentToolTip) {
+    m_currentToolTip->deleteLater();
+    m_currentToolTip = 0;
+  }
 }
 
 void ContextBrowserPlugin::startDelayedBrowsing(KTextEditor::View* view) {
@@ -224,6 +319,14 @@ void ContextBrowserPlugin::startDelayedBrowsing(KTextEditor::View* view) {
   }
 }
 
+void ContextBrowserPlugin::hideTooTip() {
+  if(m_currentToolTip) {
+    m_currentToolTip->deleteLater();
+    m_currentToolTip = 0;
+  }
+}
+
+
 void ContextBrowserPlugin::showToolTip(KTextEditor::View* view, KTextEditor::Cursor position) {
   KUrl viewUrl(view->document()->url());
   QList<ILanguage*> languages = ICore::self()->languageController()->languagesForUrl(viewUrl);
@@ -244,7 +347,7 @@ void ContextBrowserPlugin::showToolTip(KTextEditor::View* view, KTextEditor::Cur
         if(m_currentToolTipDeclaration == IndexedDeclaration(decl) && m_currentToolTip)
           return;
         m_currentToolTipDeclaration = IndexedDeclaration(decl);
-        navigationWidget = decl->context()->createNavigationWidget(decl);
+        navigationWidget = decl->context()->createNavigationWidget(decl, DUChainUtils::standardContextForUrl(viewUrl));
       }
     }
   }
@@ -261,16 +364,27 @@ void ContextBrowserPlugin::showToolTip(KTextEditor::View* view, KTextEditor::Cur
         contextView->setNavigationWidget(navigationWidget);
       }
       
-    if(m_currentToolTip)
-      delete m_currentToolTip;
+    if(m_currentToolTip) {
+      m_currentToolTip->deleteLater();
+      m_currentToolTip = 0;
+    }
     
     KDevelop::NavigationToolTip* tooltip = new KDevelop::NavigationToolTip(view, view->mapToGlobal(view->cursorToCoordinate(position)) + QPoint(20, 40), navigationWidget);
     tooltip->resize( navigationWidget->sizeHint() + QSize(10, 10) );
     kDebug() << "tooltip size" << tooltip->size();
     m_currentToolTip = tooltip;
+    ActiveToolTip::showToolTip(tooltip);
+
+    //First disconnect to prevent multiple connections
+    disconnect(view, SIGNAL(cursorPositionChanged(KTextEditor::View*,KTextEditor::Cursor)), this, SLOT(hideTooTip()));
+    disconnect(view, SIGNAL(focusOut(KTextEditor::View*)), this, SLOT(hideTooTip()));
+    connect(view, SIGNAL(cursorPositionChanged(KTextEditor::View*,KTextEditor::Cursor)), this, SLOT(hideTooTip()));
+    connect(view, SIGNAL(focusOut(KTextEditor::View*)), this, SLOT(hideTooTip()));
+    
   }else{
     kDebug() << "not showing tooltip, no navigation-widget";
   }
+  
 }
 
 
@@ -279,18 +393,39 @@ void ContextBrowserPlugin::clearMouseHover() {
   m_mouseHoverDocument.clear();
 }
 
+Attribute::Ptr highlightedUseAttribute(bool /*mouseHighlight*/, bool bold) {
+  if(bold) {
+    static Attribute::Ptr standardBoldAttribute = Attribute::Ptr();
+    if( !standardBoldAttribute ) {
+      standardBoldAttribute= Attribute::Ptr( new Attribute() );
+      standardBoldAttribute->setBackgroundFillWhitespace(true);
 
-Attribute::Ptr highlightedUseAttribute(bool /*mouseHighlight*/) {
-  static Attribute::Ptr standardAttribute = Attribute::Ptr();
-  if( !standardAttribute ) {
-    standardAttribute = Attribute::Ptr( new Attribute() );
-    standardAttribute->setBackgroundFillWhitespace(true);
-    standardAttribute->setBackground(Qt::yellow);//QApplication::palette().toolTipBase());
-    standardAttribute->setForeground(Qt::black); 
-}
-  return standardAttribute;
+      // mixing (255, 255, 0, 100) with white yields this:
+      standardBoldAttribute->setBackground(QColor(251, 250, 150));
+      standardBoldAttribute->setFontBold(true);
+
+      // force a foreground color to overwrite default Kate highlighting, i.e. of Q_OBJECT or similar
+      // foreground color could change, hence apply it everytime
+      standardBoldAttribute->setForeground(QColor(0, 0, 0, 255)); //Don't use alpha here, as kate uses the alpha only to blend with the document background color
+    }
+    return standardBoldAttribute;
+  }else{
+    static Attribute::Ptr standardAttribute = Attribute::Ptr();
+    if( !standardAttribute ) {
+      standardAttribute = Attribute::Ptr( new Attribute() );
+      standardAttribute->setBackgroundFillWhitespace(true);
+      // mixing (255, 255, 0, 100) with white yields this:
+      standardAttribute->setBackground(QColor(251, 250, 150));
+      // force a foreground color to overwrite default Kate highlighting, i.e. of Q_OBJECT or similar
+      // foreground color could change, hence apply it everytime
+      standardAttribute->setForeground(QColor(0, 0, 0, 255)); //Don't use alpha here, as kate uses the alpha only to blend with the document background color
+    }
+    return standardAttribute;
+  }
 }
 
+///NOTE: when this gets used again we have to set an alpha value as well
+/*
 Attribute::Ptr highlightedDeclarationAttribute() {
   static Attribute::Ptr standardAttribute = Attribute::Ptr();
   if( !standardAttribute ) {
@@ -300,15 +435,18 @@ Attribute::Ptr highlightedDeclarationAttribute() {
   }
   return standardAttribute;
 }
+*/
 
 Attribute::Ptr highlightedSpecialObjectAttribute() {
   static Attribute::Ptr standardAttribute = Attribute::Ptr();
   if( !standardAttribute ) {
     standardAttribute = Attribute::Ptr( new Attribute() );
     standardAttribute->setBackgroundFillWhitespace(true);
-    QColor color(Qt::yellow);
-    color.setRed(90);
-    standardAttribute->setBackground(color);
+    // mixing (90, 255, 0, 100) with white yields this:
+    standardAttribute->setBackground(QColor(190, 255, 155));
+    // force a foreground color to overwrite default Kate highlighting, i.e. of Q_OBJECT or similar
+    // foreground color could change, hence apply it everytime
+    standardAttribute->setForeground(QColor(0, 0, 0, 255)); //Don't use alpha here, as kate uses the alpha only to blend with the document background color
   }
   return standardAttribute;
 }
@@ -317,13 +455,15 @@ void ContextBrowserPlugin::changeHighlight( KTextEditor::SmartRange* range, bool
   if( !range )
     return;
 
+  QMutexLocker lock(&m_backupsMutex);
+  
   if( highlight ) {
 /*    if( declaration )
       attrib = highlightedDeclarationAttribute();
     else*/
 
-    Attribute::Ptr attrib = highlightedUseAttribute(mouseHighlight);
-    
+    Attribute::Ptr attrib = highlightedUseAttribute(mouseHighlight, range->attribute() ? range->attribute()->fontBold() : false);
+
     if( !m_backups.contains(range) ) {
       m_backups[range] = qMakePair(attrib, range->attribute());
       watchRange(range);
@@ -374,9 +514,9 @@ bool ContextBrowserPlugin::findSpecialObject(View* view, const SimpleCursor& pos
         QMutexLocker lock(iface->smartMutex());
         //Q_ASSERT(m_highlightedRange[view]->document() == view->document());
         if (m_highlightedRange[view]->document() == view->document()) {
-	  delete m_highlightedRange[view];
+          delete m_highlightedRange[view];
           m_highlightedRange.remove(view);
-      	} else {
+      } else {
           kDebug() << "m_highlightedRange[view]->document() != view->document()";	
         }
       }
@@ -384,7 +524,7 @@ bool ContextBrowserPlugin::findSpecialObject(View* view, const SimpleCursor& pos
       KUrl viewUrl = view->document()->url();
       QList<ILanguage*> languages = ICore::self()->languageController()->languagesForUrl(viewUrl);
 
-      SimpleRange r;
+      SimpleRange r = SimpleRange::invalid();
       foreach( ILanguage* language, languages) {
         r = language->languageSupport()->specialLanguageObjectRange(viewUrl, position);
         if(r.isValid()) {
@@ -407,6 +547,7 @@ bool ContextBrowserPlugin::findSpecialObject(View* view, const SimpleCursor& pos
 
 Declaration* ContextBrowserPlugin::findDeclaration(View* view, const SimpleCursor& position, bool mouseHighlight)
 {
+      Q_UNUSED(mouseHighlight);
       Declaration* foundDeclaration = 0;
       if(m_useDeclaration.data()) {
         foundDeclaration = m_useDeclaration.data();
@@ -414,23 +555,16 @@ Declaration* ContextBrowserPlugin::findDeclaration(View* view, const SimpleCurso
         //If we haven't found a special language object, search for a use/declaration and eventually highlight it
         foundDeclaration = DUChainUtils::declarationForDefinition( DUChainUtils::itemUnderCursor(view->document()->url(), position) );
       }
-      if( foundDeclaration ) {
-        m_lastHighlightedDeclaration = IndexedDeclaration(foundDeclaration);
-        m_highlightedDeclarations[view] = foundDeclaration;
-        changeHighlight( view, foundDeclaration, true, mouseHighlight );
-      }else{
-        m_highlightedDeclarations.remove(view);
-      }
       return foundDeclaration;
 }
 
 bool ContextBrowserPlugin::showDeclarationView(View* view, const SimpleCursor& position, Declaration* foundDeclaration, DUContext* ctx)
 {
+        Q_UNUSED(position);
         Q_ASSERT(foundDeclaration);
         bool success = false;
         foreach(ContextBrowserView* contextView, m_views) {
           if(masterWidget(contextView) == masterWidget(view)) {
-              contextView->updateHistory(ctx, position);
               contextView->setDeclaration(foundDeclaration, ctx->topContext());
               success = true;
           }
@@ -438,7 +572,7 @@ bool ContextBrowserPlugin::showDeclarationView(View* view, const SimpleCursor& p
         return success;
 }
 
-bool ContextBrowserPlugin::showSpecialObjectView(View* view, const SimpleCursor& position, ILanguage* pickedLanguage, DUContext* ctx)
+bool ContextBrowserPlugin::showSpecialObjectView(View* view, const SimpleCursor& position, ILanguage* pickedLanguage, DUContext* /*ctx*/)
 {
         //Q_ASSERT(pickedLanguage != 0); // specialObject was found, pickedLanguage must have been set
         if (!pickedLanguage) {
@@ -450,7 +584,6 @@ bool ContextBrowserPlugin::showSpecialObjectView(View* view, const SimpleCursor&
           if(masterWidget(contextView) == masterWidget(view)) {
               ILanguageSupport* ls = pickedLanguage->languageSupport();
               QWidget* w = ls->specialLanguageObjectNavigationWidget(view->document()->url(), position);
-              contextView->updateHistory(ctx, position);
               contextView->setSpecialNavigationWidget(w);
               success = true;
             }
@@ -460,10 +593,10 @@ bool ContextBrowserPlugin::showSpecialObjectView(View* view, const SimpleCursor&
 
 void ContextBrowserPlugin::showContextView(View* view, const SimpleCursor& position, DUContext* ctx)
 {
+    Q_UNUSED(position);
     Q_ASSERT(ctx);
     foreach(ContextBrowserView* contextView, m_views) {
       if(masterWidget(contextView) == masterWidget(view)) {
-        contextView->updateHistory(ctx, position);
         contextView->setContext(SpecializationStore::self().applySpecialization(ctx, ctx->topContext()));
       }
     }
@@ -477,8 +610,41 @@ DUContext* contextAt(const SimpleCursor& position, TopDUContext* topContext)
       return ctx;
 }
 
+
+void ContextBrowserPlugin::unHighlightAll(KTextEditor::View* selectView)
+{
+    foreach(KTextEditor::View* view, m_highlightedDeclarations.keys()) {
+      if(selectView && selectView != view)
+        continue;
+      
+      KTextEditor::SmartInterface* smart = dynamic_cast<KTextEditor::SmartInterface*>(view->document());
+    
+      QMutexLocker smartLock(smart->smartMutex());
+      QMutexLocker lock(&m_backupsMutex);
+
+      QMap< KTextEditor::SmartRange*, QPair< Attribute::Ptr, Attribute::Ptr > >::iterator it = m_backups.begin();
+      while(it != m_backups.end()) {
+        if(it.key()->document() == view->document()) {
+          if(it.key()->attribute() == it->first)
+            it.key()->setAttribute(it->second); //Set the backed up attribute, if it wasn't changed yet
+          ignoreRange(it.key());
+          it = m_backups.erase(it);
+        } else {
+          ++it;
+        }
+      }
+      
+      changeHighlight( view, m_highlightedDeclarations[view].data(), false, false );
+    }
+
+    m_highlightedDeclarations.clear();
+}
+
 void ContextBrowserPlugin::updateBrowserWidgetFor(View* view)
 {
+    bool keptHighlightedDeclaration = m_keepHighlightedDeclaration.contains(view) && m_highlightedDeclarations[view].data();
+    m_keepHighlightedDeclaration.remove(view);
+    
     bool mouseHighlight =
       (view->document()->url() == m_mouseHoverDocument) &&
       (m_mouseHoverCursor.isValid());
@@ -493,7 +659,9 @@ void ContextBrowserPlugin::updateBrowserWidgetFor(View* view)
     ///First: Check whether there is a special language object
     ///@todo Maybe make this optional, because it can be slow
     ILanguage* pickedLanguage = 0;
-    bool foundSpecialObject = findSpecialObject(view,position, pickedLanguage);
+    bool foundSpecialObject = 0;
+    if(!keptHighlightedDeclaration)
+      foundSpecialObject  = findSpecialObject(view,position, pickedLanguage);
 
     KDevelop::DUChainReadLocker lock( DUChain::lock(), 100 );
     if(!lock.locked()) {
@@ -501,15 +669,39 @@ void ContextBrowserPlugin::updateBrowserWidgetFor(View* view)
       return;
     }
 
-    if( m_highlightedDeclarations.contains(view) )     ///unhighlight the old uses
-      changeHighlight( view, m_highlightedDeclarations[view].data(), false, mouseHighlight );
+    if(!keptHighlightedDeclaration)
+      unHighlightAll();
 
     TopDUContext* topContext = DUChainUtils::standardContextForUrl(view->document()->url());
     if (!topContext) return;
     DUContext* ctx = contextAt(position, topContext);
     if (!ctx) return;
-    Declaration* foundDeclaration = (foundSpecialObject) ? 0 : findDeclaration(view, position, mouseHighlight);
-
+    Declaration* foundDeclaration = 0;
+    
+    //Only update the history if this context is around the text cursor
+    if(core()->documentController()->activeDocument() && position == SimpleCursor(view->cursorPosition()) && view->document() == core()->documentController()->activeDocument()->textDocument())
+    {
+      foreach(ContextBrowserView* contextView, m_views)
+        if(masterWidget(contextView) == masterWidget(view))
+          contextView->updateHistory(ctx, position);
+    }
+    
+    if(keptHighlightedDeclaration) {
+      foundDeclaration = m_highlightedDeclarations[view].data();
+      kDebug() << "keeping";
+    }
+    
+    if(!foundSpecialObject && !foundDeclaration)
+      foundDeclaration = findDeclaration(view, position, mouseHighlight);
+    
+    if( foundDeclaration ) {
+      m_lastHighlightedDeclaration = IndexedDeclaration(foundDeclaration);
+      m_highlightedDeclarations[view] = foundDeclaration;
+      changeHighlight( view, foundDeclaration, true, mouseHighlight );
+    }else{
+      m_highlightedDeclarations.remove(view);
+    }
+    
     bool addedWidget = false;
     // try to add a declaration navigation widget
     if(foundDeclaration || foundSpecialObject) {
@@ -518,7 +710,6 @@ void ContextBrowserPlugin::updateBrowserWidgetFor(View* view)
           addedWidget = foundSpecialObject ?
             showSpecialObjectView(view, position, pickedLanguage, ctx) :
             showDeclarationView(view, position, foundDeclaration, ctx);
-          ///@todo show tooltip
       }
     }
 
@@ -548,20 +739,47 @@ void ContextBrowserPlugin::declarationSelectedInUI(DeclarationPointer decl)
 
 void ContextBrowserPlugin::parseJobFinished(KDevelop::ParseJob* job)
 {
-  KDevelop::DUChainWriteLocker lock( DUChain::lock() );
-  registerAsRangeWatcher(job->duChain());
+  if(job->duChain() && job->duChain()->smartRange()) {
+    KDevelop::DUChainReadLocker lock( DUChain::lock() );
+    registerAsRangeWatcher(job->duChain());
+  }
+  
+  for(QMap< View*, DeclarationPointer >::const_iterator it = m_highlightedDeclarations.constBegin(); it != m_highlightedDeclarations.constEnd(); ++it) {
+    if(it.key()->document()->url() == job->document().toUrl()) {
+      if(m_updateViews.isEmpty())
+        m_updateTimer->start(highlightingTimeout);
+      
+      if(!m_updateViews.contains(it.key())) {
+        kDebug() << "adding view for update";
+        m_updateViews << it.key();
+        m_keepHighlightedDeclaration.insert(it.key());
+      }
+    }
+  }
 }
 
 void ContextBrowserPlugin::registerAsRangeWatcher(KDevelop::DUChainBase* base)
 {
-  if(base->smartRange())
+  if(base->smartRange()) {
     watchRange(base->smartRange());
 }
+}
 
 void ContextBrowserPlugin::registerAsRangeWatcher(KDevelop::DUContext* ctx)
 {
-  if(!ctx)
+  if(!ctx || !ctx->smartRange())
     return;
+
+  QMutex* lockSmartMutex = 0;
+
+  if(!ctx->parentContext()) {
+    KTextEditor::SmartInterface* smart = dynamic_cast<KTextEditor::SmartInterface*>(ctx->smartRange()->document());
+    Q_ASSERT(smart);
+    lockSmartMutex = smart->smartMutex();
+  }
+  
+  QMutexLocker lock(lockSmartMutex);
+  
   if(dynamic_cast<TopDUContext*>(ctx) && static_cast<TopDUContext*>(ctx)->parsingEnvironmentFile() && static_cast<TopDUContext*>(ctx)->parsingEnvironmentFile()->isProxyContext() && !ctx->importedParentContexts().isEmpty())
     return registerAsRangeWatcher(ctx->importedParentContexts()[0].context(0));
 
@@ -588,7 +806,7 @@ void ContextBrowserPlugin::textDocumentCreated( KDevelop::IDocument* document )
   foreach( View* view, document->textDocument()->views() )
     viewCreated( document->textDocument(), view );
 
-  KDevelop::DUChainWriteLocker lock( DUChain::lock() );
+  KDevelop::DUChainReadLocker lock( DUChain::lock() );
   QList<TopDUContext*> chains = DUChain::self()->chainsForDocument( document->url() );
 
   foreach( TopDUContext* chain, chains )
@@ -609,18 +827,35 @@ void ContextBrowserPlugin::viewDestroyed( QObject* obj )
   m_updateViews.remove(static_cast<View*>(obj));
 }
 
-void ContextBrowserPlugin::cursorPositionChanged( View* view, const KTextEditor::Cursor& /*newPosition*/ )
+void ContextBrowserPlugin::cursorPositionChanged( View* view, const KTextEditor::Cursor& newPosition )
 {
+  if(view->document() == m_lastInsertionDocument && newPosition == m_lastInsertionPos)
+  {
+    //Do not update the highlighting while typing
+    m_lastInsertionDocument = 0;
+    m_lastInsertionPos = KTextEditor::Cursor();
+    m_keepHighlightedDeclaration.insert(view);
+  }else{
+    m_keepHighlightedDeclaration.remove(view);
+  }
   clearMouseHover();
   m_updateViews.insert(view);
   m_updateTimer->start(highlightingTimeout/2); // triggers updateViews()
 }
 
+void ContextBrowserPlugin::textInserted(KTextEditor::Document* doc, KTextEditor::Range range)
+{
+  m_lastInsertionDocument = doc;
+  m_lastInsertionPos = range.end();
+}
+
 void ContextBrowserPlugin::viewCreated( KTextEditor::Document* , View* v )
 {
   disconnect( v, SIGNAL( cursorPositionChanged( KTextEditor::View*, const KTextEditor::Cursor& ) ), this, SLOT( cursorPositionChanged( KTextEditor::View*, const KTextEditor::Cursor& ) ) ); ///Just to make sure that multiple connections don't happen
   connect( v, SIGNAL( cursorPositionChanged( KTextEditor::View*, const KTextEditor::Cursor& ) ), this, SLOT( cursorPositionChanged( KTextEditor::View*, const KTextEditor::Cursor& ) ) );
   connect( v, SIGNAL(destroyed( QObject* )), this, SLOT( viewDestroyed( QObject* ) ) );
+  disconnect( v->document(), SIGNAL(textInserted(KTextEditor::Document*,KTextEditor::Range)), this, SLOT(textInserted(KTextEditor::Document*,KTextEditor::Range)));
+  connect( v->document(), SIGNAL(textInserted(KTextEditor::Document*,KTextEditor::Range)), this, SLOT(textInserted(KTextEditor::Document*,KTextEditor::Range)));
 
   KTextEditor::TextHintInterface *iface = dynamic_cast<KTextEditor::TextHintInterface*>(v);
   if( !iface )
@@ -704,7 +939,7 @@ void ContextBrowserPlugin::switchUse(bool forward)
         KDevVarLengthArray<IndexedTopDUContext> usingFiles = DUChain::uses()->uses(decl->id());
         
         if(DUChainUtils::contextHasUse(decl->topContext(), decl) && usingFiles.indexOf(decl->topContext()) == -1)
-          usingFiles.insert(decl->topContext(), 0);
+          usingFiles.insert(0, decl->topContext());
         
         if(decl->range().contains(c) && decl->url() == chosen->url()) {
             //The cursor is directly on the declaration. Jump to the first or last use.
@@ -831,6 +1066,19 @@ void ContextBrowserPlugin::unRegisterToolView(ContextBrowserView* view)
   m_views.removeAll(view);
 }
 
+QWidget* ContextBrowserPlugin::toolbarWidgetForMainWindow(QWidget* widgetInWindow)
+{
+  QWidget* master = masterWidget(widgetInWindow);
+  
+  for(QList< QPointer< QWidget > >::const_iterator it = m_toolbarWidgets.constBegin(); it != m_toolbarWidgets.constEnd(); ++it) {
+    if((*it) && masterWidget(*it) == master) {
+      return *it;
+    }
+  }
+  m_toolbarWidgets.append(new QWidget(master));
+  m_toolbarWidgets.back()->setHidden(true);
+  return m_toolbarWidgets.back();
+}
 
 #include "contextbrowser.moc"
 
