@@ -30,11 +30,13 @@
 #include <interfaces/iplugincontroller.h>
 #include <language/interfaces/ilanguagesupport.h>
 #include <language/backgroundparser/backgroundparser.h>
+#include <language/duchain/duchain.h>
 
 #include "core.h"
 #include "language.h"
 #include "settings/ccpreferences.h"
 #include "completionsettings.h"
+#include <QThread>
 
 namespace KDevelop {
 
@@ -44,7 +46,7 @@ typedef QHash<QString, QList<ILanguage*> > LanguageCache;
 
 struct LanguageControllerPrivate {
     LanguageControllerPrivate(LanguageController *controller)
-        : dataMutex(QMutex::Recursive), backgroundParser(new BackgroundParser(controller)), m_controller(controller) {}
+        : dataMutex(QMutex::Recursive), backgroundParser(new BackgroundParser(controller)), m_controller(controller), m_cleanedUp(false) {}
 
     void documentActivated(KDevelop::IDocument *document)
     {
@@ -76,6 +78,7 @@ struct LanguageControllerPrivate {
     MimeTypeCache mimeTypeCache; //Maps mimetypes to languages
 
     BackgroundParser *backgroundParser;
+    bool m_cleanedUp;
     
     ILanguage* addLanguageForSupport(ILanguageSupport* support);
 
@@ -95,7 +98,7 @@ ILanguage* LanguageControllerPrivate::addLanguageForSupport(KDevelop::ILanguageS
 
     QVariant mimetypes = Core::self()->pluginController()->pluginInfo(dynamic_cast<IPlugin*>(languageSupport)).property("X-KDevelop-SupportedMimeTypes");
 
-    foreach(QString mimeTypeName, mimetypes.toStringList()) {
+    foreach(const QString& mimeTypeName, mimetypes.toStringList()) {
         kDebug() << "adding supported mimetype:" << mimeTypeName << "language:" << languageSupport->name();
         languageCache[mimeTypeName] << ret;
         KMimeType::Ptr mime = KMimeType::mimeType(mimeTypeName);
@@ -123,10 +126,19 @@ LanguageController::~LanguageController()
 
 void LanguageController::initialize()
 {
+    // make sure the DUChain is setup before we try to access it from different threads at the same time
+    DUChain::self();
+
     connect(Core::self()->documentController(), SIGNAL(documentActivated(KDevelop::IDocument*)),
             SLOT(documentActivated(KDevelop::IDocument*)));
 }
 
+void LanguageController::cleanup()
+{
+    QMutexLocker lock(&d->dataMutex);
+    d->m_cleanedUp = true;
+}
+
 QList<ILanguage*> LanguageController::activeLanguages()
 {
     QMutexLocker lock(&d->dataMutex);
@@ -142,6 +154,10 @@ QList<ILanguage*> LanguageController::loadedLanguages() const
 {
     QMutexLocker lock(&d->dataMutex);
     QList<ILanguage*> ret;
+    
+    if(d->m_cleanedUp)
+        return ret;
+    
     foreach(ILanguage* lang, d->languages)
         ret << lang;
     return ret;
@@ -151,13 +167,15 @@ ILanguage *LanguageController::language(const QString &name) const
 {
     QMutexLocker lock(&d->dataMutex);
     
+    if(d->m_cleanedUp)
+        return 0;
+    
     if(d->languages.contains(name))
         return d->languages[name];
     else{
         ILanguage* ret = 0;
         QStringList constraints;
-        ///@todo Lookup by language-name still does not work
-        constraints << QString("'%1' in [X-KDevelop-Language]").arg(name);
+        constraints << QString("'%1' == [X-KDevelop-Language]").arg(name);
         QList<IPlugin*> supports = Core::self()->pluginController()->
             allPluginsForExtension("ILanguageSupport", constraints);
         if(!supports.isEmpty()) {
@@ -175,49 +193,61 @@ QList<ILanguage*> LanguageController::languagesForUrl(const KUrl &url)
     
     QList<ILanguage*> languages;
     
+    if(d->m_cleanedUp)
+        return languages;
+    
     QString fileName = url.fileName();
 
-    for(int a = 0; a < 2; ++a) {
-
-        ///non-crashy part: Use the mime-types of known languages
-        ///Very inefficient right now
-        for(LanguageControllerPrivate::MimeTypeCache::const_iterator it = d->mimeTypeCache.begin(); it != d->mimeTypeCache.end(); ++it) {
-            foreach(QString pattern, it.key()->patterns()) {
-                if(pattern.startsWith('*'))
+    ///TODO: evaluate on what the memory impact would be to cache the QRegExp
+    ///      for every pattern
+    ///      right now > 70% in languagesForUrl is spent in setPattern...
+    QRegExp exp("", Qt::CaseInsensitive, QRegExp::Wildcard);
+    ///non-crashy part: Use the mime-types of known languages
+    ///Very inefficient right now
+    for(LanguageControllerPrivate::MimeTypeCache::const_iterator it = d->mimeTypeCache.constBegin();
+        it != d->mimeTypeCache.constEnd(); ++it)
+    {
+        foreach(QString pattern, it.key()->patterns()) {
+            if(pattern.startsWith('*'))
                 pattern = pattern.mid(1);
 
-                QRegExp exp(pattern, Qt::CaseInsensitive, QRegExp::Wildcard);
-                if(int position = exp.indexIn(fileName)) {
-                    
-                    if(position != -1 && exp.matchedLength() + position == fileName.length())
-                        languages << *it;
-                }
+            exp.setPattern(pattern);
+            if(int position = exp.indexIn(fileName)) {
+                if(position != -1 && exp.matchedLength() + position == fileName.length())
+                    languages << *it;
             }
+        }
     }
-        
-        if(!languages.isEmpty() || a)
-            return languages;
-        
-        ///Crashy and unsafe part: Load missing language-supports
-        KMimeType::Ptr mimeType = KMimeType::findByUrl(url);
 
-        LanguageCache::iterator it = d->languageCache.find(mimeType->name());
-        if (it != d->languageCache.constEnd()) {
-            languages = it.value();
+    //Never use findByUrl from within a background thread, and never load a language support
+    //from within the backgruond thread. Both is unsafe, and can lead to crashes
+    if(!languages.isEmpty() || QThread::currentThread() != thread())
+        return languages;
+
+    ///Crashy and unsafe part: Load missing language-supports
+    KMimeType::Ptr mimeType = KMimeType::findByUrl(url);
+
+    LanguageCache::ConstIterator it = d->languageCache.constFind(mimeType->name());
+    if (it != d->languageCache.constEnd()) {
+        languages = it.value();
+    } else {
+        QStringList constraints;
+        constraints << QString("'%1' in [X-KDevelop-SupportedMimeTypes]").arg(mimeType->name());
+        QList<IPlugin*> supports = Core::self()->pluginController()->
+            allPluginsForExtension("ILanguageSupport", constraints);
+
+        if (supports.isEmpty()) {
+            d->languageCache.insert(mimeType->name(), QList<ILanguage*>());
         } else {
-            QStringList constraints;
-            constraints << QString("'%1' in [X-KDevelop-SupportedMimeTypes]").arg(mimeType->name());
-            QList<IPlugin*> supports = Core::self()->pluginController()->
-                allPluginsForExtension("ILanguageSupport", constraints);
-
             foreach (IPlugin *support, supports) {
                 ILanguageSupport* languageSupport = support->extension<ILanguageSupport>();
                 kDebug() << "language-support:" << languageSupport;
                 if(languageSupport)
-                    return languages << d->addLanguageForSupport(languageSupport);
+                    languages << d->addLanguageForSupport(languageSupport);
             }
         }
     }
+
     return languages;
 }
 
