@@ -46,6 +46,10 @@
 #include "partdocument.h"
 #include "textdocument.h"
 #include "documentcontroller.h"
+#include "assistantpopup.h"
+#include <kactioncollection.h>
+#include <ktexteditor/view.h>
+#include "workingsetcontroller.h"
 
 namespace KDevelop {
 
@@ -54,6 +58,8 @@ public:
     UiControllerPrivate(UiController *controller)
     : cfgDlg(0), areasRestored(false), m_controller(controller)
     {
+        Core::self()->workingSetControllerInternal()->initializeController(m_controller);
+
         QMap<QString, Sublime::Position> desired;
 
         desired["org.kdevelop.ClassBrowserView"] = Sublime::Left;
@@ -63,32 +69,38 @@ public:
         desired["org.kdevelop.ProblemReporterView"] = Sublime::Bottom;
         desired["org.kdevelop.OutputView"] = Sublime::Bottom;
         desired["org.kdevelop.ContextBrowser"] = Sublime::Bottom;
-        desired["org.kdevelop.debugger.ConsoleView"] = Sublime::Bottom;
+        desired["org.kdevelop.KonsoleView"] = Sublime::Bottom;
+        desired["org.kdevelop.SnippetView"] = Sublime::Right;
         Sublime::Area* a =
             new Sublime::Area(m_controller, "code", i18n("Code"));
         a->setDesiredToolViews(desired);
-        controller->addDefaultArea(a);
+        a->setIconName("document-edit");
+        m_controller->addDefaultArea(a);
 
         desired.clear();
         desired["org.kdevelop.debugger.VariablesView"] = Sublime::Left;
         desired["org.kdevelop.debugger.BreakpointsView"] = Sublime::Bottom;
         desired["org.kdevelop.debugger.StackView"] = Sublime::Bottom;
         desired["org.kdevelop.debugger.ConsoleView"] = Sublime::Bottom;
+        desired["org.kdevelop.KonsoleView"] = Sublime::Bottom;
         a = new Sublime::Area(m_controller, "debug", i18n("Debug"));
         a->setDesiredToolViews(desired);
-        controller->addDefaultArea(a);
+        a->setIconName("tools-report-bug");
+        m_controller->addDefaultArea(a);
 
         desired.clear();
-        desired["org.kdevelop.QTestTestRunner"] = Sublime::Right;
-        desired["org.kdevelop.CoverageReport"] = Sublime::Right;
-        a = new Sublime::Area(m_controller, "test", i18n("Test"));
+        desired["org.kdevelop.ProjectsView"] = Sublime::Left;
+        desired["org.kdevelop.PatchReview"] = Sublime::Bottom;
+        
+        a = new Sublime::Area(m_controller, "review", i18n("Review"));
         a->setDesiredToolViews(desired);
-        controller->addDefaultArea(a);        
+        a->setIconName("applications-engineering");
+        m_controller->addDefaultArea(a);
 
         if(!(Core::self()->setupFlags() & Core::NoUi)) 
         {
-            defaultMainWindow = new MainWindow(controller);
-            controller->addMainWindow(defaultMainWindow);
+            defaultMainWindow = new MainWindow(m_controller);
+            m_controller->addMainWindow(defaultMainWindow);
             activeSublimeWindow = defaultMainWindow;
         }
         else
@@ -119,6 +131,9 @@ public:
     QList<Sublime::MainWindow*> sublimeWindows;
     bool areasRestored;
 
+    //Currently shown assistant popup.
+    QPointer<AssistantPopup> currentShownAssistant;
+
 private:
     UiController *m_controller;
 };
@@ -141,7 +156,6 @@ public:
     }
 
     QString id() const { return m_factory->id(); }
-
 private:
     IToolViewFactory *m_factory;
 };
@@ -159,9 +173,40 @@ UiController::UiController(Core *core)
 {
     setObjectName("UiController");
     d->core = core;
+
+    if (!defaultMainWindow() || (Core::self()->setupFlags() & Core::NoUi))
+        return;
+
     connect( QApplication::instance(),
              SIGNAL( focusChanged( QWidget*, QWidget* ) ),
             this, SLOT( widgetChanged( QWidget*, QWidget* ) ) );
+
+    KActionCollection * actions = defaultMainWindow()->actionCollection();
+
+    KAction* assistantaction1 = actions->addAction("assistant_action_1");
+    assistantaction1->setText( i18n("&Assistant Action 1") );
+    assistantaction1->setShortcut( Qt::ALT | Qt::Key_1 );
+    connect(assistantaction1, SIGNAL(triggered(bool)), this, SLOT(assistantAction1()));
+
+    KAction* assistantaction2 = actions->addAction("assistant_action_2");
+    assistantaction2->setText( i18n("&Assistant Action 2") );
+    assistantaction2->setShortcut( Qt::ALT | Qt::Key_2 );
+    connect(assistantaction2, SIGNAL(triggered(bool)), this, SLOT(assistantAction2(bool)));
+
+    KAction* assistantaction3 = actions->addAction("assistant_action_3");
+    assistantaction3->setText( i18n("&Assistant Action 3") );
+    assistantaction3->setShortcut( Qt::ALT | Qt::Key_3 );
+    connect(assistantaction3, SIGNAL(triggered(bool)), this, SLOT(assistantAction3(bool)));
+
+    KAction* assistantaction4 = actions->addAction("assistant_action_4");
+    assistantaction4->setText( i18n("&Assistant Action 4") );
+    assistantaction4->setShortcut( Qt::ALT | Qt::Key_4 );
+    connect(assistantaction4, SIGNAL(triggered(bool)), this, SLOT(assistantAction4(bool)));
+
+    KAction* assistantactionhide = actions->addAction("assistant_action_hide");
+    assistantactionhide->setText( i18n("&Hide Assistant") );
+    assistantactionhide->setShortcut( Qt::ALT | Qt::Key_0 );
+    connect(assistantactionhide, SIGNAL(triggered(bool)), this, SLOT(assistantHide()));
 }
 
 UiController::~UiController()
@@ -182,7 +227,10 @@ void UiController::mainWindowDeleted(MainWindow* mw)
 // should just rename it.
 void UiController::switchToArea(const QString &areaName, SwitchMode switchMode)
 {
-    Q_UNUSED( switchMode );
+    if (switchMode == ThisWindow) {
+        showArea(areaName, activeSublimeWindow());
+        return;
+    }
 
     MainWindow *main = new MainWindow(this);
     // FIXME: what this is supposed to do?
@@ -212,6 +260,41 @@ void UiController::switchToArea(const QString &areaName, SwitchMode switchMode)
 }
 
 
+QWidget* UiController::findToolView(const QString& name, IToolViewFactory *factory, FindFlags flags)
+{
+    if(!d->areasRestored)
+        return 0;
+
+    QList< Sublime::View* > views = activeArea()->toolViews();
+    foreach(Sublime::View* view, views) {
+        Sublime::ToolDocument *doc = dynamic_cast<Sublime::ToolDocument*>(view->document());
+        if(doc && doc->title() == name && view->widget()) {
+            if(flags & Raise)
+                view->requestRaise();
+            return view->widget();
+        }
+    }
+    
+    QWidget* ret = 0;
+    
+    if(flags & Create)
+    {
+        if(!d->factoryDocuments.contains(factory))
+            d->factoryDocuments[factory] = new Sublime::ToolDocument(name, this, new UiToolViewFactory(factory));
+        
+        Sublime::ToolDocument *doc = d->factoryDocuments[factory];
+
+        Sublime::View* view = addToolViewToArea(factory, doc, activeArea());
+        if(view)
+            ret = view->widget();
+        
+        if(flags & Raise)
+            findToolView(name, factory, Raise);
+    }
+
+    return ret;
+}
+
 void UiController::addToolView(const QString & name, IToolViewFactory *factory)
 {
     kDebug() ;
@@ -292,8 +375,8 @@ void UiController::addNewToolView(MainWindow *mw)
     QListWidget *list = new QListWidget(dia);
 
     list->setSortingEnabled(true);
-    for (QMap<IToolViewFactory*, Sublime::ToolDocument*>::const_iterator it = d->factoryDocuments.begin();
-        it != d->factoryDocuments.end(); ++it)
+    for (QMap<IToolViewFactory*, Sublime::ToolDocument*>::const_iterator it = d->factoryDocuments.constBegin();
+        it != d->factoryDocuments.constEnd(); ++it)
     {
         ViewSelectorItem *item = new ViewSelectorItem(it.value()->title(), list);
         item->factory = it.key();
@@ -328,7 +411,7 @@ void UiController::showSettingsDialog()
 // the show() call doesn't show the dialog
 //     if( d->cfgDlg->dialog()->parentWidget() != activeMainWindow() )
 //         d->cfgDlg->dialog()->setParent( activeMainWindow() );
-    d->cfgDlg->show();
+    d->cfgDlg->exec();
 }
 
 Sublime::Controller* UiController::controller()
@@ -344,105 +427,14 @@ KParts::MainWindow *UiController::activeMainWindow()
 void UiController::saveArea(Sublime::Area * area, KConfigGroup & group)
 {
     area->save(group);
-    saveArea(area->rootIndex(), group);
-}
-
-void UiController::saveArea(Sublime::AreaIndex * area, KConfigGroup & group)
-{
-    if (area->isSplitted()) {
-        group.writeEntry("Orientation", area->orientation() == Qt::Horizontal ? "Horizontal" : "Vertical");
-
-        if (area->first()) {
-            KConfigGroup subgroup(&group, "0");
-            subgroup.deleteGroup();
-            saveArea(area->first(), subgroup);
-        }
-
-        if (area->second()) {
-            KConfigGroup subgroup(&group, "1");
-            subgroup.deleteGroup();
-            saveArea(area->second(), subgroup);
-        }
-    } else {
-        group.writeEntry("View Count", area->viewCount());
-
-        int index = 0;
-        foreach (Sublime::View* view, area->views()) {
-            group.writeEntry(QString("View %1 Type").arg(index), view->document()->documentType());
-            group.writeEntry(QString("View %1").arg(index), view->document()->documentSpecifier());
-            QString state = view->viewState();
-            if (!state.isEmpty())
-                group.writeEntry(QString("View %1 State").arg(index), state);
-
-            ++index;
-        }
-    }
+    Core::self()->workingSetControllerInternal()->getWorkingSet(area->workingSet())->saveFromArea(area, area->rootIndex());
 }
 
 void UiController::loadArea(Sublime::Area * area, const KConfigGroup & group)
 {
     area->load(group);
-    loadArea(area, area->rootIndex(), group);
-}
-
-void UiController::loadArea(Sublime::Area* area, Sublime::AreaIndex* areaIndex, const KConfigGroup& group)
-{
-    if (group.hasKey("Orientation")) {
-        QStringList subgroups = group.groupList();
-
-        if (subgroups.contains("0")) {
-            if (!areaIndex->isSplitted())
-                areaIndex->split(group.readEntry("Orientation", "Horizontal") == "Vertical" ? Qt::Vertical : Qt::Horizontal);
-
-            KConfigGroup subgroup(&group, "0");
-            loadArea(area, areaIndex->first(), subgroup);
-
-            if (subgroups.contains("1")) {
-                Q_ASSERT(areaIndex->isSplitted());
-                KConfigGroup subgroup(&group, "1");
-                loadArea(area, areaIndex->second(), subgroup);
-            }
-        }
-
-    } else {
-        while (areaIndex->isSplitted()) {
-            areaIndex = areaIndex->first();
-            Q_ASSERT(areaIndex);// Split area index did not contain a first child area index if this fails
-        }
-
-        int viewCount = group.readEntry("View Count", 0);
-        for (int i = 0; i < viewCount; ++i) {
-            QString type = group.readEntry(QString("View %1 Type").arg(i), "");
-            QString specifier = group.readEntry(QString("View %1").arg(i), "");
-
-            bool viewExists = false;
-            foreach (Sublime::View* view, areaIndex->views()) {
-                if (view->document()->documentSpecifier() == specifier) {
-                    viewExists = true;
-                    break;
-                }
-            }
-
-            if (viewExists)
-                continue;
-
-
-            IDocument* doc = Core::self()->documentControllerInternal()->openDocument(specifier,
-                KTextEditor::Cursor::invalid(), IDocumentController::DoNotActivate | IDocumentController::DoNotCreateView);
-            Sublime::Document *document = dynamic_cast<Sublime::Document*>(doc);
-            if (document) {
-                Sublime::View* view = document->createView();
-
-                QString state = group.readEntry(QString("View %1 State").arg(i), "");
-                if (!state.isEmpty())
-                    view->setState(state);
-
-                area->addView(view, areaIndex);
-            } else {
-                kWarning() << "Unable to create view of type " << type;
-            }
-        }
-    }
+    WorkingSet* set = Core::self()->workingSetControllerInternal()->getWorkingSet(area->workingSet());
+    Q_ASSERT(set->isConnected(area));
 }
 
 void UiController::saveAllAreas(KSharedConfig::Ptr config)
@@ -490,8 +482,8 @@ void UiController::loadAllAreas(KSharedConfig::Ptr config)
     foreach (Sublime::Area *area, defaultAreas())
     {
         QMap<IToolViewFactory*, Sublime::ToolDocument*>::const_iterator i, e;
-        for (i = d->factoryDocuments.begin(),
-                 e = d->factoryDocuments.end(); i != e; ++i)
+        for (i = d->factoryDocuments.constBegin(),
+                 e = d->factoryDocuments.constEnd(); i != e; ++i)
         {
             addToolViewIfWanted(i.key(), i.value(), area);
         }
@@ -503,6 +495,10 @@ void UiController::loadAllAreas(KSharedConfig::Ptr config)
         KConfigGroup mainWindowConfig(&uiConfig,
                                       QString("Main Window %1").arg(w));
         QString currentArea = mainWindowConfig.readEntry("currentArea", "");
+        
+        if(currentArea == "test") ///@todo The area was renamed, and it will lead to a crash, so this is a dirty temporary workaround
+            currentArea = "review";
+        
         Sublime::MainWindow *mw = mainWindows()[w];
 
         /* We loop over default areas.  This means that if
@@ -532,8 +528,8 @@ void UiController::loadAllAreas(KSharedConfig::Ptr config)
             // At this point we know which toolviews the area wants.
             // Tender all tool views we have.
             QMap<IToolViewFactory*, Sublime::ToolDocument*>::const_iterator i, e;
-            for (i = d->factoryDocuments.begin(),
-                     e = d->factoryDocuments.end(); i != e; ++i)
+            for (i = d->factoryDocuments.constBegin(),
+                     e = d->factoryDocuments.constEnd(); i != e; ++i)
             {
                 addToolViewIfWanted(i.key(), i.value(), area);
             }
@@ -552,6 +548,16 @@ void UiController::loadAllAreas(KSharedConfig::Ptr config)
     d->areasRestored = true;
 }
 
+void UiController::addToolViewToDockArea(const QString& name,
+                                         IToolViewFactory* factory,
+                                         Qt::DockWidgetArea area)
+{
+    ///TODO: we should probably add a bool forcePosition member to
+    /// Area::addToolView(), to force adding at the given position.
+    Sublime::View* view = addToolViewToArea(factory, d->factoryDocuments[factory], activeArea());
+    activeArea()->moveToolView(view, Sublime::dockAreaToPosition(area));
+}
+
 void UiController::addToolViewIfWanted(IToolViewFactory* factory,
                            Sublime::ToolDocument* doc,
                            Sublime::Area* area)
@@ -562,7 +568,7 @@ void UiController::addToolViewIfWanted(IToolViewFactory* factory,
     }
 }
 
-void UiController::addToolViewToArea(IToolViewFactory* factory,
+Sublime::View* UiController::addToolViewToArea(IToolViewFactory* factory,
                                      Sublime::ToolDocument* doc,
                                      Sublime::Area* area)
 {
@@ -575,6 +581,7 @@ void UiController::addToolViewToArea(IToolViewFactory* factory,
             SLOT(raiseToolView(Sublime::View*)));
 
     factory->viewCreated(view);
+    return view;
 }
 
 void UiController::registerStatus(QObject* status)
@@ -586,6 +593,91 @@ void UiController::registerStatus(QObject* status)
     mw->registerStatus(status);
 }
 
+void UiController::showErrorMessage(const QString& message, int timeout)
+{
+    Sublime::MainWindow* w = activeSublimeWindow();
+    if (!w) return;
+    MainWindow* mw = qobject_cast<KDevelop::MainWindow*>(w);
+    if (!mw) return;
+    QMetaObject::invokeMethod(mw, "showErrorMessage", Q_ARG(QString, message), Q_ARG(int, timeout));
 }
 
+void UiController::hideAssistant(const KDevelop::IAssistant::Ptr& assistant)
+{
+    if(d->currentShownAssistant && d->currentShownAssistant->assistant() == assistant) {
+        disconnect(d->currentShownAssistant->assistant().data(), SIGNAL(hide()), this, SLOT(assistantHide()));
+        disconnect(d->currentShownAssistant->assistant().data(), SIGNAL(actionsChanged()), this, SLOT(assistantActionsChanged()));
+        
+        AssistantPopup* oldPopup = d->currentShownAssistant;
+        d->currentShownAssistant = 0;
+        oldPopup->hide();
+        oldPopup->deleteLater(); //We have to do deleteLater, so we don't get problems when an assistant hides itself
+    }
+}
+
+void UiController::popUpAssistant(const KDevelop::IAssistant::Ptr& assistant)
+{
+    assistantHide();
+    if(!assistant)
+        return;
+    
+    Sublime::View* view = d->activeSublimeWindow->activeView();
+    if( !view )
+    {
+        kDebug() << "no active view in mainwindow";
+        return;
+    }
+    TextEditorWidget* textWidget = dynamic_cast<TextEditorWidget*>(view->widget());
+    if(textWidget && textWidget->editorView()) {
+
+        d->currentShownAssistant = new AssistantPopup(textWidget->editorView(), assistant);
+        if(assistant->actions().count())
+            d->currentShownAssistant->show();
+
+        connect(assistant.data(), SIGNAL(hide()), SLOT(assistantHide()), Qt::DirectConnection);
+        connect(assistant.data(), SIGNAL(actionsChanged()), SLOT(assistantActionsChanged()), Qt::DirectConnection);
+    }
+}
+
+const QMap< IToolViewFactory*, Sublime::ToolDocument* >& UiController::factoryDocuments() const
+{
+    return d->factoryDocuments;
+}
+
+void UiController::assistantAction1() {
+    if(d->currentShownAssistant)
+        d->currentShownAssistant->executeAction1();
+}
+
+void UiController::assistantAction2(bool) {
+    if(d->currentShownAssistant)
+        d->currentShownAssistant->executeAction2();
+}
+
+void UiController::assistantAction3(bool) {
+    if(d->currentShownAssistant)
+        d->currentShownAssistant->executeAction3();
+}
+void UiController::assistantAction4(bool) {
+    if(d->currentShownAssistant)
+        d->currentShownAssistant->executeAction4();
+}
+
+void UiController::assistantHide() {
+    if(d->currentShownAssistant)
+        hideAssistant(d->currentShownAssistant->assistant());
+}
+
+void UiController::assistantActionsChanged() {
+    if(d->currentShownAssistant)
+        popUpAssistant(d->currentShownAssistant->assistant());
+}
+
+
+}
+
+
+
+
+
 #include "uicontroller.moc"
