@@ -28,6 +28,7 @@
 #include "document.h"
 #include "areaindex.h"
 #include "controller.h"
+#include <qpointer.h>
 
 namespace Sublime {
 
@@ -48,8 +49,10 @@ struct AreaPrivate {
         toolViewPositions.clear();
         desiredToolViews = p.desiredToolViews;
         shownToolView = p.shownToolView;
+        workingSet = p.workingSet;
 
         title = p.title;
+        iconName = p.iconName;
     }
     ~AreaPrivate()
     {
@@ -89,6 +92,9 @@ struct AreaPrivate {
     QMap<QString, Sublime::Position> desiredToolViews;
     QMap<Sublime::Position, QString> shownToolView;
     QMap<Sublime::Position, int> thickness;
+    QString iconName;
+    QString workingSet;
+    QPointer<View> activeView;
 };
 
 
@@ -103,6 +109,9 @@ Area::Area(Controller *controller, const QString &name, const QString &title)
     setObjectName(name);
     d->title = title;
     d->controller = controller;
+    d->iconName = "kdevelop";
+    d->workingSet.clear();
+    kDebug() << "initial working-set:" << d->workingSet;
     initialize();
 }
 
@@ -142,13 +151,33 @@ Area::~Area()
     delete d;
 }
 
+View* Area::activeView()
+{
+    return d->activeView;
+}
+
+void Area::setActiveView(View* view)
+{
+    d->activeView = view;
+}
+
 void Sublime::Area::addView(View *view, AreaIndex *index)
 {
     index->add(view);
+    connect(view, SIGNAL(positionChanged(Sublime::View*, int)), this, SLOT(positionChanged(Sublime::View*, int)));
+    kDebug() << "view added in" << this;
     emit viewAdded(index, view);
     connect(this, SIGNAL(destroyed()), view, SLOT(deleteLater()));
 }
 
+void Sublime::Area::addViewSilently(View *view, AreaIndex *index)
+{
+    index->add(view);
+    connect(view, SIGNAL(positionChanged(Sublime::View*, int)), this, SLOT(positionChanged(Sublime::View*, int)));
+    kDebug() << "view added in" << this;
+    connect(this, SIGNAL(destroyed()), view, SLOT(deleteLater()));
+}
+
 void Area::addView(View *view, View *after)
 {
     AreaIndex *index = d->currentIndex;
@@ -164,6 +193,11 @@ void Area::addView(View *view, View *after)
 void Area::addView(View *view, View *viewToSplit, Qt::Orientation orientation)
 {
     AreaIndex *indexToSplit = indexOf(viewToSplit);
+    addView(view, indexToSplit, orientation);
+}
+
+void Area::addView(View* view, AreaIndex* indexToSplit, Qt::Orientation orientation)
+{
     indexToSplit->split(view, orientation);
     emit viewAdded(indexToSplit, view);
     connect(this, SIGNAL(destroyed()), view, SLOT(deleteLater()));
@@ -176,6 +210,8 @@ View* Area::removeView(View *view)
 
     emit aboutToRemoveView(index, view);
     index->remove(view);
+    emit viewRemoved(index, view);
+    
     return view;
 }
 
@@ -283,6 +319,7 @@ void Area::save(KConfigGroup& group) const
     group.writeEntry("thickness right", thickness(Sublime::Right));
     group.writeEntry("thickness bottom", thickness(Sublime::Bottom));
     group.writeEntry("thickness top", thickness(Sublime::Top));
+    group.writeEntry("working set", d->workingSet);
 }
 
 void Area::load(const KConfigGroup& group)
@@ -314,6 +351,7 @@ void Area::load(const KConfigGroup& group)
     setThickness(Sublime::Right, group.readEntry("thickness right", -1));
     setThickness(Sublime::Bottom, group.readEntry("thickness bottom", -1));
     setThickness(Sublime::Top, group.readEntry("thickness top", -1));
+    setWorkingSet(group.readEntry("working set", d->workingSet));
 }
 
 bool Area::wantToolView(const QString& id)
@@ -350,6 +388,68 @@ int Area::thickness(Sublime::Position pos) const
     return (d->thickness)[pos];
 }
 
+QString Area::iconName() const
+{
+    return d->iconName;
+}
+
+void Area::setIconName(const QString& iconName)
+{
+    d->iconName = iconName;
+}
+
+void Area::positionChanged(View *view, int newPos)
+{
+    AreaIndex *index = indexOf(view);
+    index->views().move(index->views().indexOf(view), newPos);
+}
+
+
+QString Area::workingSet() const
+{
+    return d->workingSet;
+}
+
+
+void Area::setWorkingSet(QString name)
+{
+    if(name != d->workingSet) {
+        kDebug() << this << "setting new working-set" << name;
+        QString oldName = d->workingSet;
+        emit changingWorkingSet(this, oldName, name);
+        d->workingSet = name;
+        emit changedWorkingSet(this, oldName, name);
+    }
+}
+
+bool Area::closeView(View* view)
+{
+    QPointer<Document> doc = view->document();
+                
+    //if(doc && doc->views().count() == 1) {    
+    if (doc && doc->uniqueView(this, view)) {
+        if(!doc->closeDocument())
+            return false;
+        else
+            return true;
+    }
+    
+    //close only one active view
+    removeView(view);
+    delete view;
+
+    if(doc && doc->views().count() == 0)
+        doc->closeDocument(); //close the document instead
+
+    return true;
+}
+
+void Area::clearViews()
+{
+    foreach(Sublime::View* view, views())
+        closeView(view);
+}
+
 }
 
 #include "area.moc"
