@@ -34,17 +34,18 @@
 #include "../vcsexport.h"
 #include "../vcsjob.h"
 
-class DVCSjobPrivate;
+class QDir;
+class DVcsJobPrivate;
 
 /**
  * This class is capable of running our dvcs commands. 
- * Most of all DVCSjobs are created in DVCS executors, but executed in DistributedVersionControlPlugin or 
+ * Most of all DVcsJob are created in DVCS executors, but executed in DistributedVersionControlPlugin or
  * any managers like BranchManager.
- * @note Connect to Kjob::result(KJob*) to be notified when the job finished.
+ * @note Connect to KJob::result(KJob*) to be notified when the job finished.
  * 
- * How to create DVCSjob:
+ * How to create DVcsJob:
  * @code
- * DVCSjob* job = new DVCSjob(vcsplugin);
+ * DVcsJob* job = new DVcsJob(vcsplugin);
  * if (job)
  * {
  *     job->setDirectory(workDir);
@@ -60,7 +61,7 @@ class DVCSjobPrivate;
  * Usage example 1:
  * @code
  * VcsJob* j = add(DistributedVersionControlPlugin::d->m_ctxUrlList, IBasicVersionControl::Recursive);
- * DVCSjob* job = dynamic_cast<DVCSjob*>(j);
+ * DVcsJob* job = dynamic_cast<DVCSjob*>(j);
  * if (job) {
  *     connect(job, SIGNAL(result(KJob*) ),
  *             this, SIGNAL(jobFinished(KJob*) ));
@@ -70,15 +71,15 @@ class DVCSjobPrivate;
  * 
  * Usage example 2:
  * @code
- * DVCSjob *branchJob = d->branch(repo, baseBranch, newBranch);
- * DVCSjob* job = gitRevParse(dirPath.path(), QStringList(QString("--is-inside-work-tree")));
+ * DVcsJob* branchJob = d->branch(repo, baseBranch, newBranch);
+ * DVcsJob* job = gitRevParse(dirPath.toLocalFile(), QStringList(QString("--is-inside-work-tree")));
  * if (job)
  * {
  *     job->exec();
  *     if (job->status() == KDevelop::VcsJob::JobSucceeded)
  *         return true;
  *     else
- *     //something, mabe even just
+ *     //something, maybe even just
  *         return false
  * }
  * @endcode
@@ -86,12 +87,12 @@ class DVCSjobPrivate;
  * @author Robert Gruber <rgruber@users.sourceforge.net>
  * @author Evgeniy Ivanov <powerfox@kde.ru>
  */
-class KDEVPLATFORMVCS_EXPORT DVCSjob : public KDevelop::VcsJob
+class KDEVPLATFORMVCS_EXPORT DVcsJob : public KDevelop::VcsJob
 {
     Q_OBJECT
 public:
-    DVCSjob(KDevelop::IPlugin* parent);
-    virtual ~DVCSjob();
+    DVcsJob(KDevelop::IPlugin* parent, KDevelop::OutputJob::OutputJobVerbosity verbosity = KDevelop::OutputJob::Verbose);
+    virtual ~DVcsJob();
 
     /**
      * Call this method to clear the job (for example, before setting another job).
@@ -105,53 +106,53 @@ public:
 
     /**
      * Sets working directory.
-     * @param directory Should contain only absolute path. Relative path or "" (working dir) are deprecated and will make job failed.
-     * @note In DVCS plugins directory variable is used to get relative pathes.
+     * @param directory Should contain only absolute path. Relative paths or "" (working dir) are deprecated and will make the job fail.
+     * @note In DVCS plugins directory variable is used to get relative paths.
      */
-    void setDirectory(const QString& directory);
+    void setDirectory(const QDir & directory);
 
     /**
-     * Sets standart Input file.
+     * Sets standard input file.
      */
     void setStandardInputFile(const QString &fileName);
 
     /**
      * Returns current working directory.
      */
-    QString getDirectory();
+    QDir const & getDirectory() const;
 
     /**
      * Call this method to set command to execute and its arguments.
      * @note Don't forget <<"one two"; is not the same as <<"one"<<"two"; Use one word(command, arg) per one QString!
      */
-    DVCSjob& operator<<(const QString& arg);
+    DVcsJob& operator<<(const QString& arg);
 
     /**
      * Overloaded convenience function.
      * @see operator<<(const QString& arg).
      */
-    DVCSjob& operator<<(const char* arg);
+    DVcsJob& operator<<(const char* arg);
 
     /**
      * Overloaded convenience function.
      * @see operator<<(const QString& arg).
      */
-    DVCSjob& operator<<(const QStringList& args);
+    DVcsJob& operator<<(const QStringList& args);
 
     /**
-     * Call this mehod to start this job.
-     * @note Default communiaction mode is KProcess::AllOutput.
+     * Call this method to start this job.
+     * @note Default communication mode is KProcess::AllOutput.
      * @see Use setCommunicationMode() to override the default communication mode.
      */
     virtual void start();
 
     /**
-     * In some cases it's needed to specify the communisation mode between the
+     * In some cases it's needed to specify the communication mode between the
      * process and the job object. This is for instance done for the "git status"
-     * command. If stdout and stderr are processed as separate streams their signals
-     * do not always get emmited in correct order by KProcess. Which will lead to a
+     * command. If stdout and stderr are processed as separate streams, their signals
+     * do not always get emitted in correct order by KProcess, which will lead to a
      * screwed up output.
-     * @note Default communiaction mode is KProcess::SeparateChannels.
+     * @note Default communication mode is KProcess::SeparateChannels.
      */
     void setCommunicationMode(KProcess::OutputChannelMode comm);
 
@@ -161,14 +162,19 @@ public:
     QString dvcsCommand() const;
 
     /**
-     * @return The whole output of the job.
+     * @return The whole output of the job as a string. (Might fail on binary data)
      */
     QString output() const;
 
+    /**
+     * @return The whole binary output of the job
+     */
+    QByteArray rawOutput() const;
+
     // Begin:  KDevelop::VcsJob
 
     /** 
-     * Sets executions reults.
+     * Sets executions results.
      * In most cases this method is used by IDVCSexecutor
      * @see fetchResults()
      */
@@ -184,8 +190,8 @@ public:
     /**
      * Sets exit status (d->failed variable).
      * Since only executors can parse the job to set result, they can connect parsers to readyForParsing(DVCSjob) using
-     * Qt::DirectConnection to set the result. For example git-status can return exit status 1 
-     * if you don't set exit status in your parser then you will have JobFailes in status() result.
+     * Qt::DirectConnection to set the result. For example git-status can return exit status 1.
+     * If you don't set exit status in your parser then you will have JobFailed in status() result.
      * @note First result is set in slotProcessExited() or slotProcessError().
      */
     virtual void setExitStatus(const bool exitStatus);
@@ -216,17 +222,17 @@ public Q_SLOTS:
     bool isRunning() const;
 
 Q_SIGNALS:
-    void readyForParsing(DVCSjob *job);
+    void readyForParsing(DVcsJob *job);
 
 private Q_SLOTS:
     void slotProcessError( QProcess::ProcessError );
     void slotProcessExited(int exitCode, QProcess::ExitStatus exitStatus);
-    void slotReceivedStdout(const QStringList&);
-    void slotReceivedStderr(const QStringList&);
+    void slotReceivedStdout();
+    void slotReceivedStderr();
 
 private:
     void jobIsReady();
-    DVCSjobPrivate* const d;
+    DVcsJobPrivate* const d;
     QVariant results;
 };
 
